# vizpaint/sankey_diagram.py
"""
桑基图（Sankey Diagram）模块
用于展示流量、能量或资源流动的图表
"""

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.sankey import Sankey


def sankey_diagram(flows, labels=None, orientations=0,
                   pathlengths=0.25, trunklength=1.0,
                   head_angle=100, margin=0.4,
                   title="Sankey Diagram", color_palette='Set3'):
    """
    绘制桑基图（Sankey Diagram）

    参数:
        flows: 流量列表，正值为向右，负值为向左
              例如: [0.5, 0.3, -0.2, 0.1, -0.7]
        labels: 标签列表，长度应为 flows.length + 1
        orientations: 流向方向列表（0: 右，1: 上，-1: 下）
        pathlengths: 路径长度
        trunklength: 主干长度
        head_angle: 箭头角度
        margin: 边距
        title: 图表标题
        color_palette: 颜色调色板名称

    返回:
        fig, ax, sankey: 图形、坐标轴和桑基图对象
    """
    # 创建图形
    fig, ax = plt.subplots(figsize=(14, 10))

    # 设置标签（如果未提供）
    if labels is None:
        labels = [f'Node {i}' for i in range(len(flows) + 1)]

    # 设置方向（如果未提供）
    if isinstance(orientations, int):
        orientations = [orientations] * (len(flows) - 1)

    # 创建桑基图对象
    sankey = Sankey(ax=ax, scale=1.0, offset=0.1,
                    head_angle=head_angle,
                    trunklength=trunklength,
                    margin=margin)

    # 添加主要流动
    sankey.add(flows=flows,
               labels=labels,
               orientations=orientations,
               pathlengths=pathlengths,
               facecolor=plt.cm.get_cmap(color_palette)(0.3))

    # 完成图表
    diagrams = sankey.finish()

    # 自定义每个箭头的颜色
    cmap = plt.cm.get_cmap(color_palette)
    for i, diagram in enumerate(diagrams):
        for j, patch in enumerate(diagram.patches):
            # 为每个路径分配不同的颜色
            color = cmap(j / max(len(diagram.patches) - 1, 1))
            patch.set_facecolor(color)
            patch.set_alpha(0.8)
            patch.set_edgecolor('white')
            patch.set_linewidth(1)

    # 添加标签到每个节点
    for diagram in diagrams:
        for text in diagram.texts:
            text.set_fontsize(10)
            text.set_fontweight('bold')

    # 设置标题
    ax.set_title(title, fontsize=18, fontweight='bold', pad=20)

    # 移除坐标轴
    ax.axis('off')

    # 添加图例说明
    total_flow = sum(abs(f) for f in flows if f > 0)
    info_text = f'总流量: {total_flow:.2f} 单位\n箭头宽度表示流量大小'

    props = dict(boxstyle='round', facecolor='lightgray', alpha=0.7)
    ax.text(0.02, 0.02, info_text, transform=ax.transAxes,
            fontsize=10, verticalalignment='bottom',
            bbox=props)

    plt.tight_layout()
    return fig, ax, sankey


def create_simple_sankey():
    """创建示例桑基图数据"""
    # 示例：能源流动
    flows = [0.5, 0.3, -0.2, 0.1, -0.7, 0.2, -0.2]
    labels = ['能源生产', '发电', '输电损耗',
              '工业用电', '居民用电', '商业用电', '损耗']

    fig, ax, sankey = sankey_diagram(
        flows=flows,
        labels=labels,
        orientations=[0, 1, 0, -1, 1, 0],  # 方向控制
        title="能源分配桑基图",
        color_palette='Set2'
    )

    return fig, ax, sankey


def create_budget_sankey():
    """创建预算分配桑基图"""
    flows = [1.0, -0.4, -0.3, -0.2, -0.1,  # 第一层分配
             0.4, -0.2, -0.1, -0.1,  # 运营费用细分
             0.3, -0.15, -0.10, -0.05]  # 研发费用细分

    labels = ['总预算', '运营', '研发', '市场', '其他',
              '运营预算', '人力', '设备', '办公',
              '研发预算', '新产品', '技术升级', '测试']

    fig, ax, sankey = sankey_diagram(
        flows=flows,
        labels=labels,
        title="年度预算分配桑基图",
        color_palette='tab20c',
        margin=0.3
    )

    return fig, ax, sankey


"""
堆叠面积图
"""

import matplotlib.pyplot as plt
from vizpaint.utils import set_chinese_font
import numpy as np


def create_stacked_area_chart(data, x_column, y_columns, title="堆叠面积图",
                              xlabel="X轴", ylabel="Y轴", figsize=(12, 8),
                              colors=None, legend_loc='upper left', alpha=0.7,
                              rotation=45):
    """
    创建堆叠面积图

    参数:
    ----------
    data : DataFrame
        包含数据的DataFrame
    x_column : str
        X轴列名
    y_columns : list
        要堆叠显示的Y轴列名列表
    title : str, optional
        图表标题
    xlabel : str, optional
        X轴标签
    ylabel : str, optional
        Y轴标签
    figsize : tuple, optional
        图表尺寸
    colors : list, optional
        颜色列表
    legend_loc : str, optional
        图例位置
    alpha : float, optional
        透明度
    rotation : int, optional
        X轴标签旋转角度

    返回:
    ----------
    fig, ax : 图表对象和坐标轴对象
    """
    set_chinese_font()
    fig, ax = plt.subplots(figsize=figsize)

    if colors is None:
        colors = plt.cm.Set3(np.linspace(0, 1, len(y_columns)))

    # 创建堆叠面积图
    ax.stackplot(data[x_column], [data[col] for col in y_columns],
                 labels=y_columns, colors=colors, alpha=alpha)

    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    ax.set_title(title, fontsize=16, fontweight='bold')
    ax.legend(loc=legend_loc)
    ax.grid(True, alpha=0.3)

    # 旋转X轴标签
    if rotation != 0:
        plt.setp(ax.get_xticklabels(), rotation=rotation, ha='right')

    plt.tight_layout()
    return fig, ax


"""核心功能模块"""


class CodeSubmitter:
    """代码提交管理器"""

    def __init__(self, repo_url=None):
        self.repo_url = repo_url
        self.submissions = []

    def validate_submission(self, code, contributor_name):
        """验证提交的代码"""
        # 基础验证逻辑
        if not code or not contributor_name:
            return False, "代码和贡献者名称不能为空"

        # 添加更多验证规则
        if len(code) > 10000:
            return False, "代码过长"

        return True, "验证通过"

    def add_submission(self, code, contributor_name, description=""):
        """添加提交记录"""
        is_valid, message = self.validate_submission(code, contributor_name)

        if is_valid:
            submission = {
                'code': code,
                'contributor': contributor_name,
                'description': description,
                'timestamp': self._get_timestamp()
            }
            self.submissions.append(submission)
            return True, "提交成功"
        else:
            return False, message

    def _get_timestamp(self):
        from datetime import datetime
        return datetime.now().isoformat()


def process_data(data, processor_func=None):
    """处理数据，可以接受自定义处理函数"""
    if processor_func and callable(processor_func):
        return processor_func(data)
    return data


def main_function():
    """主功能函数"""
    return "Welcome to My Python Module!"


"""
堆叠柱状图
"""

import numpy as np
import matplotlib.pyplot as plt
from vizpaint.utils import set_chinese_font


def create_stacked_bar_chart(data, x_column, y_columns, title="堆叠柱状图",
                             xlabel="X轴", ylabel="Y轴", figsize=(12, 8),
                             colors=None, legend_loc='upper right', show_values=False,
                             rotation=45):
    """
    创建堆叠柱状图

    参数:
    ----------
    data : DataFrame
        包含数据的DataFrame
    x_column : str
        X轴列名
    y_columns : list
        要堆叠显示的Y轴列名列表
    title : str, optional
        图表标题
    xlabel : str, optional
        X轴标签
    ylabel : str, optional
        Y轴标签
    figsize : tuple, optional
        图表尺寸
    colors : list, optional
        颜色列表
    legend_loc : str, optional
        图例位置
    show_values : bool, optional
        是否在柱状图上显示数值
    rotation : int, optional
        X轴标签旋转角度

    返回:
    ----------
    fig, ax : 图表对象和坐标轴对象
    """
    set_chinese_font()
    fig, ax = plt.subplots(figsize=figsize)

    x = np.arange(len(data[x_column]))
    bottom = np.zeros(len(data[x_column]))

    if colors is None:
        colors = plt.cm.Set3(np.linspace(0, 1, len(y_columns)))

    for i, col in enumerate(y_columns):
        bars = ax.bar(x, data[col], bottom=bottom, label=col,
                      color=colors[i], edgecolor='black', alpha=0.8)
        bottom += data[col].values

        if show_values:
            for j, bar in enumerate(bars):
                height = bar.get_height()
                if height > 0:
                    ax.text(bar.get_x() + bar.get_width() / 2.,
                            bottom[j] - height / 2,
                            f'{height:.1f}', ha='center', va='center',
                            fontsize=9, color='white' if height > bottom[j] / 2 else 'black')

    ax.set_xticks(x)
    ax.set_xticklabels(data[x_column], rotation=rotation)
    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    ax.set_title(title, fontsize=16, fontweight='bold')
    ax.legend(loc=legend_loc)
    ax.grid(axis='y', alpha=0.3)

    plt.tight_layout()
    return fig, ax


"""
分组箱线图
"""

import matplotlib.pyplot as plt
import seaborn as sns
from vizpaint.utils import set_chinese_font

import numpy as np
def create_grouped_boxplot(data, x_column, y_column, group_column=None,
                           title="分组箱线图", xlabel="X轴", ylabel="Y轴",
                           figsize=(12, 8), colors=None, legend_loc='upper right',
                           rotation=45):
    """
    创建分组箱线图

    参数:
    ----------
    data : DataFrame
        包含数据的DataFrame
    x_column : str
        X轴列名
    y_column : str
        Y轴列名
    group_column : str, optional
        分组列名
    title : str, optional
        图表标题
    xlabel : str, optional
        X轴标签
    ylabel : str, optional
        Y轴标签
    figsize : tuple, optional
        图表尺寸
    colors : list, optional
        颜色列表
    legend_loc : str, optional
        图例位置
    rotation : int, optional
        X轴标签旋转角度

    返回:
    ----------
    fig, ax : 图表对象和坐标轴对象
    """
    set_chinese_font()
    fig, ax = plt.subplots(figsize=figsize)

    if group_column is None:
        # 简单箱线图
        if colors is None:
            colors = plt.cm.Set3(0.3)

        ax.boxplot([data[data[x_column] == cat][y_column].dropna()
                    for cat in data[x_column].unique()])

        ax.set_xticklabels(data[x_column].unique(), rotation=rotation)

    else:
        # 分组箱线图
        if colors is None:
            colors = plt.cm.Set3(np.linspace(0, 1, len(data[group_column].unique())))

        # 使用seaborn创建分组箱线图
        sns.boxplot(x=x_column, y=y_column, hue=group_column,
                    data=data, ax=ax, palette=colors)

        # 旋转X轴标签
        if rotation != 0:
            plt.setp(ax.get_xticklabels(), rotation=rotation, ha='right')

    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    ax.set_title(title, fontsize=16, fontweight='bold')
    ax.legend(loc=legend_loc)
    ax.grid(True, alpha=0.3)

    plt.tight_layout()
    return fig, ax


"""
分组柱状图（复式柱状图）
"""

import numpy as np
import matplotlib.pyplot as plt
from vizpaint.utils import set_chinese_font


def create_grouped_bar_chart(data, x_column, y_columns, group_column=None,
                             title="分组柱状图", xlabel="X轴", ylabel="Y轴",
                             figsize=(12, 8), colors=None, legend_loc='upper right',
                             bar_width=0.8, show_values=False, rotation=0):
    """
    创建分组柱状图

    参数:
    ----------
    data : DataFrame
        包含数据的DataFrame
    x_column : str
        X轴列名
    y_columns : list
        要显示的Y轴列名列表（当group_column为None时使用）
    group_column : str, optional
        分组列名，如果提供，则根据该列分组显示多个系列
    title : str, optional
        图表标题
    xlabel : str, optional
        X轴标签
    ylabel : str, optional
        Y轴标签
    figsize : tuple, optional
        图表尺寸
    colors : list, optional
        颜色列表
    legend_loc : str, optional
        图例位置
    bar_width : float, optional
        柱状图宽度
    show_values : bool, optional
        是否在柱状图上显示数值
    rotation : int, optional
        X轴标签旋转角度

    返回:
    ----------
    fig, ax : 图表对象和坐标轴对象
    """
    set_chinese_font()
    fig, ax = plt.subplots(figsize=figsize)

    if group_column is None:
        # 简单分组柱状图
        x = np.arange(len(data[x_column]))
        bar_width = bar_width / len(y_columns)

        if colors is None:
            colors = plt.cm.Set3(np.linspace(0, 1, len(y_columns)))

        for i, col in enumerate(y_columns):
            offset = (i - len(y_columns) / 2 + 0.5) * bar_width
            bars = ax.bar(x + offset, data[col], width=bar_width,
                          label=col, color=colors[i], edgecolor='black', alpha=0.8)

            if show_values:
                for bar in bars:
                    height = bar.get_height()
                    ax.text(bar.get_x() + bar.get_width() / 2., height + 0.01,
                            f'{height:.1f}', ha='center', va='bottom', fontsize=9)

        ax.set_xticks(x)
        ax.set_xticklabels(data[x_column], rotation=rotation)

    else:
        # 复杂分组柱状图（基于group_column分组）
        groups = data[group_column].unique()
        x_categories = data[x_column].unique()

        x = np.arange(len(x_categories))
        bar_width = bar_width / len(groups)

        if colors is None:
            colors = plt.cm.Set3(np.linspace(0, 1, len(groups)))

        for i, group in enumerate(groups):
            group_data = data[data[group_column] == group]
            values = []
            for cat in x_categories:
                subset = group_data[group_data[x_column] == cat]
                if len(subset) > 0:
                    values.append(subset[y_columns[0]].values[0])
                else:
                    values.append(0)

            offset = (i - len(groups) / 2 + 0.5) * bar_width
            bars = ax.bar(x + offset, values, width=bar_width,
                          label=group, color=colors[i], edgecolor='black', alpha=0.8)

            if show_values:
                for bar in bars:
                    height = bar.get_height()
                    if height > 0:
                        ax.text(bar.get_x() + bar.get_width() / 2., height + 0.01,
                                f'{height:.1f}', ha='center', va='bottom', fontsize=9)

        ax.set_xticks(x)
        ax.set_xticklabels(x_categories, rotation=rotation)

    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    ax.set_title(title, fontsize=16, fontweight='bold')
    ax.legend(loc=legend_loc)
    ax.grid(axis='y', alpha=0.3)

    plt.tight_layout()
    return fig, ax


"""
实用工具函数
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns


def set_chinese_font():
    """设置中文字体支持"""
    plt.rcParams['font.sans-serif'] = ['SimHei', 'Arial Unicode MS', 'DejaVu Sans']
    plt.rcParams['axes.unicode_minus'] = False
    sns.set_style("whitegrid")





"""
热力图
"""

import matplotlib.pyplot as plt
import seaborn as sns
from vizpaint.utils import set_chinese_font


def create_heatmap(data, columns=None, title="热力图",
                   figsize=(12, 10), cmap='RdYlBu_r',
                   annot=True, fmt=".2f", cbar=True,
                   square=False):
    """
    创建热力图

    参数:
    ----------
    data : DataFrame 或 相关矩阵
        包含数据的DataFrame或相关矩阵
    columns : list, optional
        要计算相关的列名列表
    title : str, optional
        图表标题
    figsize : tuple, optional
        图表尺寸
    cmap : str, optional
        颜色映射
    annot : bool, optional
        是否在热力图上显示数值
    fmt : str, optional
        数值格式
    cbar : bool, optional
        是否显示颜色条
    square : bool, optional
        是否使热力图单元格为正方形

    返回:
    ----------
    fig, ax : 图表对象和坐标轴对象
    """
    set_chinese_font()
    fig, ax = plt.subplots(figsize=figsize)

    if columns is not None:
        # 如果指定了列，则计算这些列的相关矩阵
        corr_matrix = data[columns].corr()
    else:
        # 否则直接使用data作为相关矩阵
        corr_matrix = data

    # 创建热力图
    sns.heatmap(corr_matrix, annot=annot, fmt=fmt, cmap=cmap,
                ax=ax, cbar=cbar, square=square,
                linewidths=0.5, linecolor='white')

    ax.set_title(title, fontsize=16, fontweight='bold')

    plt.tight_layout()
    return fig, ax


# vizpaint/rose_chart_enhanced.py
"""
增强版玫瑰图模块
提供动画、交互和统计功能的玫瑰图
"""

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.patches import Wedge
from matplotlib.animation import FuncAnimation
import matplotlib.colors as mcolors


def rose_chart_enhanced(values, categories=None, colors=None,
                        title="Enhanced Rose Chart",
                        animation=False, duration=3,
                        show_stats=True, sort_by_value=True,
                        highlight_top=3, explode=False):
    """
    增强版玫瑰图，支持动画和统计功能

    参数:
        values: 数值列表
        categories: 类别标签列表
        colors: 颜色列表，可选
        title: 图表标题
        animation: 是否启用生长动画
        duration: 动画时长（秒）
        show_stats: 是否显示统计信息
        sort_by_value: 是否按值排序
        highlight_top: 高亮前N个最大的扇形
        explode: 是否分离突出显示

    返回:
        fig, ax, wedges: 图形、坐标轴和扇形对象
        如果启用动画，额外返回 anim 对象
    """
    # 数据预处理
    values = np.array(values)
    n = len(values)

    if categories is None:
        categories = [f'Cat {i + 1}' for i in range(n)]

    # 排序（可选）
    if sort_by_value:
        sort_idx = np.argsort(values)[::-1]  # 降序
        values = values[sort_idx]
        categories = [categories[i] for i in sort_idx]

    # 颜色设置
    if colors is None:
        colors = plt.cm.Set3(np.linspace(0, 1, n))

    # 高亮设置
    if highlight_top > 0 and highlight_top < n:
        highlight_colors = list(colors)
        for i in range(min(highlight_top, n)):
            # 将前几个颜色加深
            rgb = mcolors.to_rgb(highlight_colors[i])
            highlight_colors[i] = tuple(min(c * 0.7, 1) for c in rgb)
        colors = highlight_colors

    # 爆炸效果
    if explode:
        explode_values = [0.05 if i < highlight_top else 0 for i in range(n)]
    else:
        explode_values = [0] * n

    # 创建图形
    fig, ax = plt.subplots(figsize=(12, 10), subplot_kw=dict(polar=True))

    # 计算角度和半径
    theta = np.linspace(0, 2 * np.pi, n, endpoint=False)
    width = 2 * np.pi / n
    radii = values / values.max() * 0.8  # 归一化半径

    # 绘制扇形
    wedges = []
    for i in range(n):
        wedge = Wedge((0, 0), radii[i],
                      np.degrees(theta[i]),
                      np.degrees(theta[i] + width),
                      facecolor=colors[i],
                      edgecolor='white',
                      linewidth=1.5,
                      alpha=0.85)
        ax.add_patch(wedge)
        wedges.append(wedge)

        # 添加标签
        label_angle = theta[i] + width / 2
        label_radius = radii[i] * 0.6
        ax.text(label_angle, label_radius, categories[i],
                ha='center', va='center',
                fontsize=10, fontweight='bold',
                rotation=np.degrees(label_angle) - 90)

        # 添加数值标签
        value_radius = radii[i] * 0.85
        ax.text(label_angle, value_radius, f'{values[i]:.1f}',
                ha='center', va='center',
                fontsize=9, color='white',
                fontweight='bold')

    # 添加标题
    ax.set_title(title, fontsize=18, fontweight='bold', pad=30)

    # 隐藏极坐标网格
    ax.grid(False)
    ax.set_xticklabels([])
    ax.set_yticklabels([])

    # 添加统计信息（可选）
    if show_stats:
        stats_text = (f'总数: {values.sum():.1f}\n'
                      f'均值: {values.mean():.2f}\n'
                      f'最大: {values.max():.1f}\n'
                      f'最小: {values.min():.1f}')

        # 在左上角添加文本框
        props = dict(boxstyle='round', facecolor='wheat', alpha=0.8)
        ax.text(0.02, 0.98, stats_text, transform=ax.transAxes,
                fontsize=10, verticalalignment='top',
                bbox=props)

    plt.tight_layout()

    # 动画效果（可选）
    if animation:
        def init():
            for wedge in wedges:
                wedge.set_radius(0)
            return wedges

        def update(frame):
            progress = frame / 100
            for i, wedge in enumerate(wedges):
                target_radius = radii[i]
                current_radius = target_radius * progress
                wedge.set_radius(current_radius)
            return wedges

        frames = int(duration * 30)  # 30 fps
        anim = FuncAnimation(fig, update, frames=frames,
                             init_func=init, blit=True,
                             interval=duration * 1000 / frames)

        return fig, ax, wedges, anim

    return fig, ax, wedges


"""vizpaint - 专业数据可视化库"""

__version__ = "0.1.8"
__author__ = "zhouxinjun"

# ==================== 基础图表 ====================
from .pie_chart import pie_chart
from .bar_chart import bar_chart
from .curve_statistical_chart import curve_statistical_chart
from .scatter_plot import scatter_plot

# ==================== 2D 统计图 ====================
from .line_plot import line_plot
from .area_chart import area_chart
from .bubble_chart import bubble_chart
from .stacked_bar_chart import stacked_bar_chart
from .histogram import histogram
from .box_plot import box_plot
from .violin_plot import violin_plot
from .radar_chart import radar_chart

from .pair_plot import pair_plot

# ==================== 玫瑰图系列 ====================
from .rose_chart import rose_chart, stacked_rose_chart, wind_rose, create_sample_wind_data
from .rose_chart_enhanced import rose_chart_enhanced

# ==================== 3D 图表 ====================
from .surface_3d import surface_3d
from .scatter_3d import scatter_3d

# ==================== 场/矢量图 ====================
from .quiver_plot import quiver_plot
from .stream_plot import stream_plot

# ==================== 高级图表 ====================
from .sankey_diagram import sankey_diagram, create_simple_sankey, create_budget_sankey
from .treemap import treemap  # 需要安装 squarify

# ==================== 复合图表（来自 compound 目录）====================
from .combination_chart import create_combination_chart
from .grouped_bar import create_grouped_bar_chart
from .grouped_boxplot import create_grouped_boxplot
from .multiline import create_multiline_chart
from .scatter_matrix import create_scatter_matrix
from .stacked_area import create_stacked_area_chart
from .stacked_bar import create_stacked_bar_chart
from .heatmap import create_heatmap  # 注意：与上面 heatmap 同名，建议重命名或别名

# ==================== 工具函数 ====================
from .utils import set_chinese_font
import matplotlib.pyplot as plt

def set_style(style='default'):
    try:
        plt.style.use(style)
    except:
        plt.style.use('default')

def show_all():
    plt.show()

def clear_all():
    plt.close('all')

def get_version():
    return __version__

# ==================== 额外实用函数 ====================
from .core import CodeSubmitter, process_data, main_function
from .chicken import chicken  # 趣味函数

# ==================== 定义 __all__ ====================
__all__ = [
    # 基础
    'pie_chart', 'bar_chart', 'curve_statistical_chart', 'scatter_plot',
    # 2D统计
    'line_plot', 'area_chart', 'bubble_chart', 'stacked_bar_chart',
    'histogram', 'box_plot', 'violin_plot', 'radar_chart', 'heatmap', 'pair_plot',
    # 玫瑰图
    'rose_chart', 'stacked_rose_chart', 'wind_rose', 'create_sample_wind_data',
    'rose_chart_enhanced',
    # 3D
    'surface_3d', 'scatter_3d',
    # 场图
    'quiver_plot', 'stream_plot',
    # 高级
    'sankey_diagram', 'create_simple_sankey', 'create_budget_sankey', 'treemap',
    # 复合图表
    'create_combination_chart', 'create_grouped_bar_chart', 'create_grouped_boxplot',
    'create_multiline_chart', 'create_scatter_matrix', 'create_stacked_area_chart',
    'create_stacked_bar_chart', 'create_heatmap',
    # 工具
    'set_style', 'show_all', 'clear_all', 'get_version', 'set_chinese_font',
    # 核心工具
    'CodeSubmitter', 'process_data', 'main_function', 'chicken'
]

# 清理不需要在顶层暴露的模块名称（可选）
del plt


"""
组合图（多种图表组合）
"""

import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
from vizpaint.utils import set_chinese_font
import numpy as np

def create_combination_chart(data, x_column, y_columns, title="组合图表",
                             figsize=(15, 12), colors=None):
    """
    创建组合图表（包含多种图表类型）

    参数:
    ----------
    data : DataFrame
        包含数据的DataFrame
    x_column : str
        X轴列名
    y_columns : list
        Y轴列名列表
    title : str, optional
        图表标题
    figsize : tuple, optional
        图表尺寸
    colors : list, optional
        颜色列表

    返回:
    ----------
    fig : 图表对象
    """
    set_chinese_font()

    if colors is None:
        colors = plt.cm.Set3(np.linspace(0, 1, len(y_columns)))

    fig = plt.figure(figsize=figsize)

    # 创建2x2的网格
    gs = GridSpec(2, 2, figure=fig)

    # 1. 多线图
    ax1 = fig.add_subplot(gs[0, 0])
    for i, col in enumerate(y_columns):
        ax1.plot(data[x_column], data[col], 'o-',
                 label=col, color=colors[i], linewidth=2)
    ax1.set_title('多线图')
    ax1.set_xlabel(x_column)
    ax1.set_ylabel('数值')
    ax1.legend(loc='upper left')
    ax1.grid(True, alpha=0.3)
    plt.setp(ax1.get_xticklabels(), rotation=45, ha='right')

    # 2. 分组柱状图
    ax2 = fig.add_subplot(gs[0, 1])
    x = range(len(data[x_column]))
    bar_width = 0.8 / len(y_columns)
    for i, col in enumerate(y_columns):
        offset = (i - len(y_columns) / 2 + 0.5) * bar_width
        ax2.bar([pos + offset for pos in x], data[col],
                width=bar_width, label=col, color=colors[i], alpha=0.8)
    ax2.set_title('分组柱状图')
    ax2.set_xlabel(x_column)
    ax2.set_ylabel('数值')
    ax2.set_xticks(x)
    ax2.set_xticklabels(data[x_column], rotation=45)
    ax2.legend(loc='upper right')
    ax2.grid(axis='y', alpha=0.3)

    # 3. 堆叠面积图
    ax3 = fig.add_subplot(gs[1, 0])
    ax3.stackplot(data[x_column], [data[col] for col in y_columns],
                  labels=y_columns, colors=colors, alpha=0.7)
    ax3.set_title('堆叠面积图')
    ax3.set_xlabel(x_column)
    ax3.set_ylabel('数值')
    ax3.legend(loc='upper left')
    ax3.grid(True, alpha=0.3)
    plt.setp(ax3.get_xticklabels(), rotation=45, ha='right')

    # 4. 散点图矩阵（简化版）
    ax4 = fig.add_subplot(gs[1, 1])
    # 只选择前两个系列绘制散点图
    if len(y_columns) >= 2:
        scatter = ax4.scatter(data[y_columns[0]], data[y_columns[1]],
                              c=data[y_columns[0]] if len(y_columns) > 2 else colors[0],
                              s=100, alpha=0.7, cmap='viridis')
        ax4.set_xlabel(y_columns[0])
        ax4.set_ylabel(y_columns[1])
        if len(y_columns) > 2:
            plt.colorbar(scatter, ax=ax4, label=y_columns[0])
    ax4.set_title('散点图')
    ax4.grid(True, alpha=0.3)

    fig.suptitle(title, fontsize=18, fontweight='bold', y=0.98)
    plt.tight_layout()

    return fig


"""
散点矩阵图
"""

import matplotlib.pyplot as plt
import seaborn as sns
from vizpaint.utils import set_chinese_font


def create_scatter_matrix(data, columns, hue_column=None, title="散点矩阵图",
                          figsize=(15, 15), diag_kind='hist', markers='o',
                          alpha=0.7, palette='Set2'):
    """
    创建散点矩阵图

    参数:
    ----------
    data : DataFrame
        包含数据的DataFrame
    columns : list
        要包含在散点矩阵中的列名列表
    hue_column : str, optional
        用于分组的列名
    title : str, optional
        图表标题
    figsize : tuple, optional
        图表尺寸
    diag_kind : str, optional
        对角线图表类型 ('hist' 或 'kde')
    markers : str, optional
        标记样式
    alpha : float, optional
        透明度
    palette : str, optional
        调色板

    返回:
    ----------
    fig : 图表对象
    """
    set_chinese_font()

    # 创建散点矩阵图
    if hue_column is not None:
        pairplot = sns.pairplot(data[columns + [hue_column]],
                                hue=hue_column, diag_kind=diag_kind,
                                markers=markers, palette=palette,
                                plot_kws={'alpha': alpha})
    else:
        pairplot = sns.pairplot(data[columns], diag_kind=diag_kind,
                                plot_kws={'alpha': alpha})

    fig = pairplot.fig
    fig.set_size_inches(figsize)
    fig.suptitle(title, fontsize=16, fontweight='bold', y=1.02)

    plt.tight_layout()
    return fig


from setuptools import setup, find_packages

# 读取 README.md（如果不存在则用默认文本）
try:
    with open("README.md", "r", encoding="utf-8") as f:
        long_description = f.read()
except FileNotFoundError:
    long_description = "A comprehensive Python data visualization library."

setup(
    name="vizpaint",
    version="0.1.9",
    author="zhouxinjun",
    author_email="369013027@qq.com",
    description="A comprehensive Python data visualization library.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://pypi.org/project/vizpaint/",
    packages=["vizpaint"],  # 只包含 vizpaint 包
    install_requires=["matplotlib>=3.3.0", "numpy>=1.19.0"],
    python_requires=">=3.7",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
